/*
  alignSeqs.c

  $Id: alignSeqs.c,v 1.1.1.1 2005/08/04 14:32:29 mcschatz Exp $

  construct a tiling of reads
*/

#include "getCoverage.h"
#define DEBUG 0

int TILINGWINDOW = 10000;
int min(int a, int b)
{
  return (a<b)?a:b;
}

void alignSeqs(struct CONTIG_SEQUENCE **bcseq,
               struct TILING_DATA *tdata,
               int numOfSeqs,
               int contigLen)
{
  // Just allocate for later use
  tdata->idcov = (unsigned int **) newmem2(TILINGWINDOW, sizeof(unsigned int *),
                                           __FILE__,__LINE__);

  tdata->cdc = (unsigned short *) newmem2(TILINGWINDOW, sizeof(unsigned short),
                                           __FILE__,__LINE__);

  tdata->dcov = (unsigned short *)newmem2(TILINGWINDOW,sizeof(unsigned short),
                                          __FILE__,__LINE__);

  tdata->numOfSeqs = numOfSeqs;
}


int loadTilingChunk(int alignStart,
                    struct TILING_DATA *td,
                    struct CONTIG_SEQUENCE **bcseq,
                    struct CONTIG_SEQUENCE * contig,
                    char doLoadQV)
{
  int alignEnd = min(alignStart+TILINGWINDOW-1, contig->len-1);
  int count;
  int i;
  int contigLen = contig->len;
  int firstReadId = -1;
  int lastReadId = -1;

  int tilingpos;
  int index;
  int maxDepth = 0;

  int numOfSeqs = td->numOfSeqs;

  pool_flush();

  if (DEBUG)
  {
    fprintf(stderr, "loading tiling chunk [%d, %d]\n",
            alignStart, alignEnd);
  }

  memset(td->dcov, 0, TILINGWINDOW * sizeof(unsigned short));
  memset(td->cdc,  0, TILINGWINDOW * sizeof(unsigned short));

  for (count = 0; count < numOfSeqs; count++)
  {
    if (hasOverlap(alignStart, alignEnd,
                   bcseq[count]->offset,
                   bcseq[count]->len,
                   contigLen))
    {
      int bcseqPos, gAsmPos;
      int bcseqLen = bcseq[count]->len;
      int offset   = bcseq[count]->offset;

      if (DEBUG)
      {
        fprintf(stderr, "Tiling %s (%d) [%d,%d]\n",
                bcseq[count]->id,
                count,
                bcseq[count]->offset,
                bcseq[count]->offset + bcseq[count]->len);
      }
      
      for (bcseqPos=0,
           gAsmPos = (offset<0) ? ((contigLen + offset) % contigLen) : offset;
           bcseqPos < bcseqLen;
           gAsmPos = ((gAsmPos+1)%contigLen), bcseqPos++)
      {
        if (gAsmPos >= alignStart && gAsmPos <= alignEnd)
        {
          tilingpos = gAsmPos - alignStart;
          td->dcov[tilingpos]++;
        }
      }

      if (firstReadId == -1)
      {
        firstReadId = count;
      }

      lastReadId = count;
    }
  }

  for (i = 0; i < TILINGWINDOW; i++)
  {
    int depth = td->dcov[i];
    td->idcov[i] = (unsigned int *) pool_malloc(depth * sizeof(unsigned int));
    if (depth > maxDepth)
    {
      maxDepth = depth;
    }
  }

  td->maxDepth = maxDepth;

  if (firstReadId != -1)
  {
    for (count = firstReadId; count <= lastReadId; count++)
    {
      if (hasOverlap(alignStart, alignEnd,
                     bcseq[count]->offset,
                     bcseq[count]->len,
                     contigLen))
      {
        int bcseqPos, gAsmPos;
        int bcseqLen = bcseq[count]->len;
        int offset   = bcseq[count]->offset;

        for (bcseqPos=0,
             gAsmPos = (offset<0) ? ((contigLen + offset) % contigLen) : offset;
             bcseqPos < bcseqLen;
             gAsmPos = ((gAsmPos+1)%contigLen), bcseqPos++)
        {
          if (gAsmPos >= alignStart && gAsmPos <= alignEnd)
          {
            tilingpos = gAsmPos-alignStart;

            index = td->cdc[tilingpos];
            td->idcov[tilingpos][index] = count;
            td->cdc[tilingpos]++;
          }
        }

        loadSeq(bcseq[count]);
        if (doLoadQV) { loadQual(bcseq[count], contig->id); }
      }
    }
  }

  return alignEnd;
}

